<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\ExtViSualMerchandiser\Model\Sorting;

use FiloBlu\ExtViSualMerchandiser\Helper\ConfigHelper;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\DB\Select;
use Magento\Store\Model\ScopeInterface;

class OutStockBottom extends \Magento\VisualMerchandiser\Model\Sorting\OutStockBottom
{
    /**
     * @param Collection $collection
     * @return Collection
     */
    public function sort(
        Collection $collection
    ) {
        if (!$this->moduleManager->isEnabled('Magento_CatalogInventory')) {
            return $collection;
        }

        if ($this->getStockId() == Stock::DEFAULT_STOCK_ID) {
            $collection->joinField(
                'is_in_stock',
                'cataloginventory_stock_item',
                'is_in_stock',
                'product_id=entity_id',
                ['stock_id' => $this->getStockId()],
                'left'
            );
        } else {
            $collection->joinField(
                'is_in_stock',
                'inventory_stock_' . $this->getStockId(),
                'is_salable',
                'sku=sku',
                null,
                'left'
            );
        }

        $collection->getSelect()
            ->reset(Select::ORDER)
            ->order('is_in_stock ' . $collection::SORT_ORDER_DESC);

        return $collection;
    }

    /**
     * @return int
     */
    protected function getStockId()
    {
        $stock = $this->scopeConfig->getValue(ConfigHelper::CONFIG_DEFAULT_STOCK, ScopeInterface::SCOPE_STORE);
        if ($stock) {
            return $stock;
        }
        return Stock::DEFAULT_STOCK_ID;
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return __("Move out of stock to bottom");
    }
}
